local addonName, addonTable = ...
WhoHasWhat = LibStub("AceAddon-3.0"):NewAddon("WhoHasWhat", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceTimer-3.0", "AceSerializer-3.0", "AceBucket-3.0")
local AceComm = LibStub("AceComm-3.0")
local Cache = LibStub('LibItemCache-2.0')
local AceGUI = LibStub('AceGUI-3.0')

function WhoHasWhat:OnInitialize()
  WhoHasWhat:RegisterChatCommand('whw', 'HandleChatCommand');

  WhoHasWhat:RegisterEvent('ADDON_LOADED', 'AddonLoaded'); 
  WhoHasWhat:RegisterBucketEvent('BAG_UPDATE', 60, 'SendBagUpdate')

  WhoHasWhat:RegisterComm("WHWMe");
  WhoHasWhat:RegisterComm("WHWQuery");
  WhoHasWhat:RegisterComm("WHWGuildData");
end

function WhoHasWhat:AddonLoaded(variable)
  if GuildData == nil then
    print('Initializing GuildData');
    GuildData = {
      {
        name = UnitName('player'),
        timestamp = time(),
        bagItems = {},
        professions = {},
        bankItems = {}
    }
  }
  end
  SelectedGuildMember = GuildData[1]
  AceComm:SendCommMessage("WHWQuery", "Querying", "GUILD");
end

function WhoHasWhat:SendBagUpdate()
  local bags = WhoHasWhat:GetBags()
  local bagItems = WhoHasWhat:GetBagItems()

  local characterData = {
    name = UnitName('player'),
    class = UnitClass('player'),
    timestamp = time(),
    bagItems = {},
    bankItems = {},
    professions = {}
  };

  for i=1, #bagItems do
      table.insert(characterData.bagItems, {ID = bagItems[i].itemID, Count = bagItems[i].count})
  end

  for i=0, GetNumSkillLines(), 1 do
    local skillName, header, isExpanded, skillRank = GetSkillLineInfo(i)
    -- expand the header if necessary
    if ( header and not isExpanded ) then
      MyExpandedHeaders[i] = skillName
    end
  end
  
  ExpandSkillHeader(0)
  for i=1, GetNumSkillLines(), 1 do

    local skillName, header, _, skillRank = GetSkillLineInfo(i)
    --print(skillRank, '', skillName)
    -- check for the skill name
    if(header and skillName == 'Professions') then
      for profession=1,GetNumPrimaryProfessions() do
        local skillName, header, _, skillRank, _, _, maxSkillRank = GetSkillLineInfo(i+profession)
        local professionData = {
          name = skillName,
          rank = skillRank,
          maxRank = maxSkillRank
        }
        table.insert(characterData.professions, professionData)
      end
    elseif(header and skillName == 'Secondary Skills') then
      for profession=1,3 do
        local skillName, header, _, skillRank, _, _, maxSkillRank = GetSkillLineInfo(i+profession)
        if(skillName ~= "Cooking" and skillName ~= "Fishing" and skillName ~= "First Aid") then
          break
        end
        local professionData = {
          name = skillName,
          rank = skillRank,
          maxRank = maxSkillRank
        }
        table.insert(characterData.professions, professionData)
      end
    end
  end
  AceComm:SendCommMessage("WHWMe", WhoHasWhat:Serialize(characterData), "GUILD");
end

function WhoHasWhat:OnCommReceived(prefix, message, distribution, sender)
  if prefix == "WHWMe" then
    -- print("Received user data from: " .. sender)
    local success, characterData = WhoHasWhat:Deserialize(message);
    local shouldAdd = true;

    for i=1, #GuildData do
      if(GuildData[i].name == characterData.name) then
        GuildData[i] = characterData;
        shouldAdd = false;
      end
    end

    if shouldAdd then 
      table.insert(GuildData, characterData);
    end
  elseif prefix == "WHWQuery" and sender ~= UnitName('player') then
    AceComm:SendCommMessage("WHWGuildData", WhoHasWhat:Serialize(GuildData), "GUILD");
  elseif prefix == "WHWGuildData" then
    local success, receivedGuildData = WhoHasWhat:Deserialize(message);
    local totalUpdate = 0;
    for i=1,#receivedGuildData do
      for j=1, #GuildData do
        if GuildData[j].name == receivedGuildData[i].name and GuildData[j].timestamp < receivedGuildData[i].timestamp then
            GuildData[j] = receivedGuildData[i];
            totalUpdate = totalUpdate + 1;
            break;
        end
      end
    end
  end
end

function SelectGroup(frame, event, group)
  frame:ReleaseChildren()
  
  if group == "inventory" then
    frame:ReleaseChildren();
    guildMembersDropdown:SetCallback('OnValueChanged', function(sender, event, value)
      SelectedGuildMember = GuildData[value]
      RenderInventory(scroll);
  end )

    scrollcontainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
    scrollcontainer:SetFullWidth(true)
    scrollcontainer:SetFullHeight(true) -- probably?
    scrollcontainer:SetLayout("Fill") -- important!
    
    frame:AddChild(scrollcontainer)
    
    scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow") -- probably?
    scrollcontainer:AddChild(scroll)

    RenderInventory(scroll);
  elseif group == "professions" then
    frame:ReleaseChildren();
    guildMembersDropdown:SetCallback('OnValueChanged', function(sender, event, value)
      SelectedGuildMember = GuildData[value]
      RenderProfessions(scroll);
  end )

    scrollcontainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
    scrollcontainer:SetFullWidth(true)
    scrollcontainer:SetFullHeight(true) -- probably?
    scrollcontainer:SetLayout("Fill") -- important!
    
    frame:AddChild(scrollcontainer)
    
    scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow") -- probably?
    scrollcontainer:AddChild(scroll)

    RenderProfessions(scroll);
  end
end

function RenderInventory(parent)
  parent:ReleaseChildren();

  if SelectedGuildMember.bagItems == nil then
    return
  end

  if SelectedGuildMember ~= nil then
    for i=1,#SelectedGuildMember.bagItems do
      local item = SelectedGuildMember.bagItems[i];
      local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,
      itemEquipLoc, itemIcon, itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, 
      isCraftingReagent = GetItemInfo(item.ID) 

      if itemName ~= nil then
        local action_icon = AceGUI:Create("ActionSlotItem")
        action_icon:SetWidth(70)
        action_icon:SetHeight(50)
        action_icon:SetText(item.ID)
        action_icon:SetActionText(nil, 'x' .. item.Count)
        action_icon:SetCallback("OnEnter", function(widget)
          GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPRIGHT");
          --GameTooltip:SetInventoryItemByID(item.ID)
          GameTooltip:SetHyperlink(itemLink);
          GameTooltip:Show();
        end)

        action_icon:SetCallback("OnLeave", function(widget)
          GameTooltip:Hide()
        end)
        parent:AddChild(action_icon)
      end
    end
  end
end

function RenderProfessions(parent)
  parent:ReleaseChildren();

  if SelectedGuildMember.professions == nil then
    return
  end

  if SelectedGuildMember ~= nil then
    for i=1,#SelectedGuildMember.professions do
      local professionLine = AceGUI:Create("SimpleGroup")
      professionLine:SetLayout("Flow")
      professionLine:SetFullWidth(true)
      professionLine:SetHeight(50)
      
      local profession = SelectedGuildMember.professions[i]
      local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo(profession.name)

      local action_icon = AceGUI:Create("ActionSlotSpell")
      action_icon:SetWidth(70)
      action_icon:SetHeight(50)
      action_icon:SetText(spellId)
      action_icon:SetActionText(nil, '')
      local professionLabel = AceGUI:Create('Label')
    
      professionLabel:SetText(profession.name .. ' - ' .. profession.rank .. '/' .. profession.maxRank)
      professionLine:AddChild(action_icon)
      professionLine:AddChild(professionLabel)

      parent:AddChild(professionLine)
    end
  end
end

function WhoHasWhat:HandleChatCommand(input)
  if input == '' then
    guildName, guildRankName, guildRankIndex = GetGuildInfo('player');
    -- WHWFrame:AddChild(action_icon);
    local frame = AceGUI:Create("Frame")
    
    frame:SetTitle("Who Has What")
    frame:SetStatusText('<' .. guildName .. '> Guild Information')
    frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
    frame:SetLayout("Flow")

    local headerBar = AceGUI:Create("SimpleGroup")
    headerBar.width = "fill"
    headerBar:SetLayout("flow")

    frame:AddChild(headerBar)

    guildMembersDropdown = AceGUI:Create("Dropdown")
    guildMembersDropdown:SetLabel("Select character:")
    guildMembersDropdown:SetWidth(200)
    local guildMembers = {}

    for i=1,#GuildData do
      table.insert(guildMembers, GuildData[i].name)
    end
    

    guildMembersDropdown:SetList(guildMembers)
    headerBar:AddChild(guildMembersDropdown)

    local tab =  AceGUI:Create("TabGroup")
    tab:SetFullWidth(true)
    tab:SetFullHeight(true)
    tab:SetLayout("Flow")
    -- Setup which tabs to show
    tab:SetTabs({{text="Inventory", value="inventory"}, {text="Professions", value="professions"}})
    -- Register callback
    tab:SetCallback("OnGroupSelected", SelectGroup)
    -- Set initial Tab (this will fire the OnGroupSelected callback)
    tab:SelectTab("inventory")

    frame:AddChild(tab)

    guildMembersDropdown:SetValue(1)
  elseif input == 'query' then
    AceComm:SendCommMessage("WHWQuery", "please", "GUILD");
  elseif input == 'clear' then
    GuildData = {}
  else 
    print('WhoHasWhat does not recognize that command.')
  end 
end

function WhoHasWhat:GetBags()
  local bags = {}

  for container = -1, 12 do
    bags[#bags + 1] = {
      container = container,
      bagName = GetBagName(container)
    }
  end

  return bags;
end

function WhoHasWhat:GetBagItems()
  local bagItems = {}

  for container = -1, 12 do
    local numSlots = GetContainerNumSlots(container)
    for slot=1, numSlots do
      local texture, count, locked, quality, readable, lootable, link, isFiltered, hasNoValue, itemID = GetContainerItemInfo(container, slot);
        if itemID ~= nil then
          bagItems[#bagItems + 1] = {                    
          container = container,
          slot = slot,
          itemID = itemID,
          count = count
        }
      end
    end
  end

  return bagItems
end